# **********************************************************************************************************************
#
#                             Copyright (c) 2019-2021 David Briant. All rights reserved.
#
# BSD 3-Clause License
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
#    following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
#    following disclaimer in the documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
#    products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# **********************************************************************************************************************

from bones.core.sentinels import Missing
from bones.lang.types import unary

# keep the contexts on the kernel to relieve the burden of type memory management from the storage manager

class Kernel(object):
    def __init__(self, sm):
        self.ctxs = {}
        self.sm = sm
        self.modByPath = {}
        self.styleByName = {}

    def styleForName(self, name):
        return self.styleByName.get(name, unary)

    def loadModules(self, paths):
        # i.e. searches PYTHON_PATH and BONES_PATH for bones/ex/ and load core.py or core.b
        for path in paths:
            root = __import__(path)
            names = path.split(".")
            modPath = names[0]
            mod = root
            if modPath not in self.modByPath:
                self.modByPath[modPath] = mod
            for name in names[1:]:
                modPath = modPath + '.' + name if modPath else name
                if modPath not in self.modByPath:
                    mod = getattr(mod, name)
                    self.modByPath[modPath] = mod


kernelForCoppertop = Kernel(Missing)
